#  Drakkar-Software OctoBot-Tentacles-Manager
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import os.path as path

import octobot_commons.json_util as json_util


def read_config(config_file: str, raise_errors: bool = True) -> dict:
    if path.exists(config_file):
        return json_util.read_file(config_file, raise_errors=raise_errors, on_error_value={})
    return {}


def write_config(config_file: str, content: dict) -> None:
    json_util.safe_dump(content, config_file)


def is_same_config(config_file: str, content: dict) -> bool:
    return json_util.has_same_content(config_file, content)



#  Drakkar-Software OctoBot-Tentacles-Manager
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import os
from os.path import isfile
from shutil import rmtree
from unittest import mock

import aiohttp
import pytest
from os import path

import octobot_tentacles_manager.configuration as configuration
import octobot_tentacles_manager.api as api
from octobot_tentacles_manager.configuration.tentacle_configuration import get_config, update_config, \
    factory_reset_config, get_config_schema_path
import octobot_tentacles_manager.util as util
import octobot_tentacles_manager.constants as constants
from octobot_tentacles_manager.loaders.tentacle_loading import reload_tentacle_by_tentacle_class

# All test coroutines will be treated as marked.
pytestmark = pytest.mark.asyncio


async def test_get_config():
    _cleanup()
    async with aiohttp.ClientSession() as session:
        await api.install_all_tentacles(_tentacles_local_path(), aiohttp_session=session)
    from tentacles.Evaluator.RealTime import InstantFluctuationsEvaluator
    setup_config = configuration.TentaclesSetupConfiguration()
    assert get_config(setup_config, InstantFluctuationsEvaluator) == {
        "price_difference_threshold_percent": 1,
        "volume_difference_threshold_percent": 400
    }
    from tentacles.Services import RedditService
    assert get_config(setup_config, RedditService) == {}
    _cleanup()


async def test_update_config():
    async with aiohttp.ClientSession() as session:
        await api.install_all_tentacles(_tentacles_local_path(), aiohttp_session=session)
    from tentacles.Evaluator.RealTime import InstantFluctuationsEvaluator
    setup_config = configuration.TentaclesSetupConfiguration()
    config_update = {
        "price_difference_threshold_percent": 2,
        "plop": 42
    }
    update_config(setup_config, InstantFluctuationsEvaluator, config_update)
    assert get_config(setup_config, InstantFluctuationsEvaluator) == {
        "price_difference_threshold_percent": 2,
        "volume_difference_threshold_percent": 400,
        "plop": 42
    }
    _cleanup()


async def test_keep_existing_update_config():
    async with aiohttp.ClientSession() as session:
        await api.install_all_tentacles(_tentacles_local_path(), aiohttp_session=session)
    from tentacles.Evaluator.RealTime import InstantFluctuationsEvaluator
    setup_config = configuration.TentaclesSetupConfiguration()
    # init nested config
    config_update = {
        "price_difference_threshold_percent": 2,
        "plop": 42,
        "nested_thing": {
            "price_difference_threshold_percent": 2,
            "plop": 42,
            "another_nested_thing": {
                "price_difference_threshold_percent": 2,
                "plop": 42
            }
        }
    }
    update_config(setup_config, InstantFluctuationsEvaluator, config_update)
    assert get_config(setup_config, InstantFluctuationsEvaluator) == {
        "price_difference_threshold_percent": 2,
        "volume_difference_threshold_percent": 400,
        "plop": 42,
        "nested_thing": {
            "price_difference_threshold_percent": 2,
            "plop": 42,
            "another_nested_thing": {
                "price_difference_threshold_percent": 2,
                "plop": 42
            }
        }
    }
    
    # test keep existing option
    config_update = {
        "nested_thing": {
            "new_other_nested_thing": {
                "price_difference_threshold_percent": 2,
                "plop": 42
            }
        }
    }
    update_config(setup_config, InstantFluctuationsEvaluator, config_update, keep_existing=True)
    assert get_config(setup_config, InstantFluctuationsEvaluator) == {
        "price_difference_threshold_percent": 2,
        "volume_difference_threshold_percent": 400,
        "plop": 42,
        "nested_thing": {
            "price_difference_threshold_percent": 2,
            "plop": 42,
            "another_nested_thing": {
                "price_difference_threshold_percent": 2,
                "plop": 42
            },
            "new_other_nested_thing": {
                "price_difference_threshold_percent": 2,
                "plop": 42
            }
        }
    }
    # test deep nested with keep existing option
    config_update = {
        "nested_thing": {
            "new_other_nested_thing": {
                "i am very deep": {
                    "price_difference_threshold_percent": 2,
                    "plop": 42
                }
            }
        }
    }
    update_config(setup_config, InstantFluctuationsEvaluator, config_update, keep_existing=True)
    assert get_config(setup_config, InstantFluctuationsEvaluator) == {
        "price_difference_threshold_percent": 2,
        "volume_difference_threshold_percent": 400,
        "plop": 42,
        "nested_thing": {
            "price_difference_threshold_percent": 2,
            "plop": 42,
            "another_nested_thing": {
                "price_difference_threshold_percent": 2,
                "plop": 42
            },
            "new_other_nested_thing": {
                "price_difference_threshold_percent": 2,
                "plop": 42,
                "i am very deep": {
                    "price_difference_threshold_percent": 2,
                    "plop": 42
                }
            }
        }
    }
    # try adding to deep config
    config_update = {
        "nested_thing": {
            "new_other_nested_thing": {
                "i am also deep": {
                    "price_difference_threshold_percent": 2,
                    "plop": 42
                }
            }
        }
    }
    update_config(setup_config, InstantFluctuationsEvaluator, config_update, keep_existing=True)
    assert get_config(setup_config, InstantFluctuationsEvaluator) == {
        "price_difference_threshold_percent": 2,
        "volume_difference_threshold_percent": 400,
        "plop": 42,
        "nested_thing": {
            "price_difference_threshold_percent": 2,
            "plop": 42,
            "another_nested_thing": {
                "price_difference_threshold_percent": 2,
                "plop": 42
            },
            "new_other_nested_thing": {
                "price_difference_threshold_percent": 2,
                "plop": 42,
                "i am very deep": {
                    "price_difference_threshold_percent": 2,
                    "plop": 42
                },
                "i am also deep": {
                    "price_difference_threshold_percent": 2,
                    "plop": 42
                }
            }
        }
    }
    
    # test keep existing false
    config_update = {
        "nested_thing": {
            "i am alone here": {
                "price_difference_threshold_percent": 42,
            }
        }
    }
    update_config(setup_config, InstantFluctuationsEvaluator, config_update, keep_existing=False)
    assert get_config(setup_config, InstantFluctuationsEvaluator) == {
        "price_difference_threshold_percent": 2,
        "volume_difference_threshold_percent": 400,
        "plop": 42,
        "nested_thing": {
            "i am alone here": {
                "price_difference_threshold_percent": 42,
            }
        }
    }
    _cleanup()


async def test_factory_reset_config():
    async with aiohttp.ClientSession() as session:
        await api.install_all_tentacles(_tentacles_local_path(), aiohttp_session=session)
    from tentacles.Evaluator.RealTime import InstantFluctuationsEvaluator
    setup_config = configuration.TentaclesSetupConfiguration()
    config_update = {
        "price_difference_threshold_percent": 2,
        "plop": 42
    }
    update_config(setup_config, InstantFluctuationsEvaluator, config_update)
    reload_tentacle_by_tentacle_class()
    factory_reset_config(setup_config, InstantFluctuationsEvaluator)
    assert get_config(setup_config, InstantFluctuationsEvaluator) == {
        "price_difference_threshold_percent": 1,
        "volume_difference_threshold_percent": 400
    }
    _cleanup()


async def test_fill_tentacle_config():
    async with aiohttp.ClientSession() as session:
        await api.install_all_tentacles(_tentacles_local_path(), aiohttp_session=session)

    setup_config = configuration.TentaclesSetupConfiguration()
    available_tentacle = util.load_tentacle_with_metadata(constants.TENTACLES_PATH)
    with mock.patch.object(setup_config, "_get_installation_context_bot_version", mock.Mock()) as bot_version_mock:
        bot_version_mock.return_value = "1.0.5"
        setup_config.fill_tentacle_config(available_tentacle, constants.TENTACLE_CONFIG_FILE_NAME)
        assert setup_config.installation_context == {
            constants.TENTACLE_INSTALLATION_CONTEXT_OCTOBOT_VERSION: "1.0.5"
        }

    setup_config = configuration.TentaclesSetupConfiguration()
    setup_config.fill_tentacle_config(available_tentacle, constants.TENTACLE_CONFIG_FILE_NAME)
    assert setup_config.installation_context == {
        constants.TENTACLE_INSTALLATION_CONTEXT_OCTOBOT_VERSION:
            constants.TENTACLE_INSTALLATION_CONTEXT_OCTOBOT_VERSION_UNKNOWN
    }

    assert not api.are_tentacles_up_to_date(setup_config,
                                            constants.TENTACLE_INSTALLATION_CONTEXT_OCTOBOT_VERSION_UNKNOWN)
    assert not api.are_tentacles_up_to_date(setup_config, '1.0.0')
    setup_config.installation_context[constants.TENTACLE_INSTALLATION_CONTEXT_OCTOBOT_VERSION] = '2.0.0'
    assert not api.are_tentacles_up_to_date(setup_config, '2.1.0')
    assert api.are_tentacles_up_to_date(setup_config, '2.0.0')
    assert api.are_tentacles_up_to_date(setup_config, '2.0.0b1')
    _cleanup()


async def test_get_config_schema_path():
    async with aiohttp.ClientSession() as session:
        await api.install_all_tentacles(_tentacles_local_path(), aiohttp_session=session)
    from tentacles.Evaluator.RealTime import InstantFluctuationsEvaluator
    assert isfile(get_config_schema_path(InstantFluctuationsEvaluator))
    _cleanup()


def _tentacles_local_path():
    return path.join("tests", "static", "tentacles.zip")


def _cleanup():
    if path.exists(constants.TENTACLES_PATH):
        rmtree(constants.TENTACLES_PATH)
    if path.exists(constants.TENTACLE_CONFIG_FILE_NAME):
        os.remove(constants.TENTACLE_CONFIG_FILE_NAME)
    if path.exists(constants.USER_REFERENCE_TENTACLE_CONFIG_PATH):
        rmtree(constants.USER_REFERENCE_TENTACLE_CONFIG_PATH)



#  Drakkar-Software OctoBot-Tentacles-Manager
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import aiofiles
import zipfile
import os
import os.path as path
import shutil

import octobot_commons.logging as commons_logging
import octobot_commons.aiohttp_util as aiohttp_util
import octobot_tentacles_manager.constants as constants
import octobot_tentacles_manager.util as util

DOWNLOADED_DATA_CHUNK_SIZE = 60000


async def fetch_and_extract_tentacles(tentacles_temp_dir, tentacles_path_or_url, aiohttp_session, merge_dirs=False):
    compressed_file = tentacles_path_or_url
    should_download = _is_url(tentacles_path_or_url)
    try:
        if should_download:
            if aiohttp_session is None:
                raise RuntimeError("Missing aiohttp_session argument")
            compressed_file = f"downloaded_{tentacles_temp_dir}"
            last_modified = await _download_tentacles(compressed_file, tentacles_path_or_url, aiohttp_session)
            await util.log_tentacles_file_details(compressed_file, last_modified)
        await _extract_tentacles(compressed_file, tentacles_temp_dir, merge_dirs)
    except Exception as err:
        commons_logging.get_logger("TentaclesFetching").warning(
            f"Error when fetching tentacles from {tentacles_path_or_url}: {err} ({err.__class__.__name__})"
        )
        raise
    finally:
        if should_download and path.isfile(compressed_file):
            os.remove(compressed_file)


def cleanup_temp_dirs(target_path):
    if path.exists(target_path):
        shutil.rmtree(target_path)


def get_local_arch_download_path():
    return f"{util.get_os_str()}/{util.get_arch_str()}"


async def _download_tentacles(target_file, download_URL, aiohttp_session):
    async with aiofiles.open(target_file, 'wb+') as downloaded_file:
        return await aiohttp_util.download_stream_file(
            output_file=downloaded_file,
            file_url=download_URL,
            aiohttp_session=aiohttp_session,
            data_chunk_size=DOWNLOADED_DATA_CHUNK_SIZE,
            is_aiofiles_output_file=True
        )


async def _extract_tentacles(source_path, target_path, merge_dirs):
    if path.exists(target_path) and path.isdir(target_path) and not merge_dirs:
        shutil.rmtree(target_path)
    with zipfile.ZipFile(source_path) as zipped_tentacles:
        for archive_member in zipped_tentacles.namelist():
            if _is_tentacle_valid_tentacle_file(archive_member):
                zipped_tentacles.extract(archive_member, target_path)


def _is_tentacle_valid_tentacle_file(archive_member):
    member_path = archive_member.split("/")
    return len(member_path) >= 2 \
        and member_path[0] == constants.TENTACLES_ARCHIVE_ROOT \
        and (member_path[1] in constants.TENTACLE_TYPES or member_path[1] == constants.TENTACLES_PACKAGE_PROFILES_PATH)


def _is_url(string):
    return string.startswith("https://") \
           or string.startswith("http://") \
           or string.startswith("ftp://") \
           or string.startswith("sftp://")



#  Drakkar-Software OctoBot-Tentacles-Manager
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import packaging.version as packaging_version

import octobot_tentacles_manager.constants as constants
import octobot_tentacles_manager.loaders as loaders
import octobot_tentacles_manager.managers as managers
import octobot_tentacles_manager.util as util


def load_tentacles(verbose=True) -> bool:
    return managers.TentaclesSetupManager.is_tentacles_arch_valid(verbose=verbose)


def is_tentacles_architecture_valid() -> bool:
    return managers.TentaclesSetupManager.is_tentacles_arch_valid(verbose=False, import_tentacles=False)


def are_tentacles_up_to_date(tentacles_setup_config, bot_version):
    installation_version = get_tentacles_installation_version(tentacles_setup_config)
    if installation_version == constants.TENTACLE_INSTALLATION_CONTEXT_OCTOBOT_VERSION_UNKNOWN:
        return False
    return packaging_version.parse(bot_version) <= packaging_version.parse(installation_version)


def get_tentacles_installation_version(tentacles_setup_config):
    return tentacles_setup_config.installation_context.get(
        constants.TENTACLE_INSTALLATION_CONTEXT_OCTOBOT_VERSION,
        constants.TENTACLE_INSTALLATION_CONTEXT_OCTOBOT_VERSION_UNKNOWN)


def is_tentacles_setup_config_successfully_loaded(tentacles_setup_config):
    return tentacles_setup_config.is_successfully_loaded


def reload_tentacle_info() -> None:
    loaders.reload_tentacle_by_tentacle_class()


def ensure_tentacle_info() -> None:
    loaders.ensure_tentacles_metadata(constants.TENTACLES_PATH)


def register_extra_tentacle_data(tentacle_class, tentacle_type_path: str):
    """
    :param tentacle_class: class of the tentacle
    :param tentacle_type_path: path of the tentacle, ex "Trading/mode"
    """
    return util.register_extra_tentacle_data(tentacle_class.get_name(), tentacle_type_path, tentacle_class)



#  Drakkar-Software OctoBot-Tentacles-Manager
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import os.path as path
import sys

import octobot_commons.tentacles_management as tentacles_management

import octobot_tentacles_manager.api as api
import octobot_tentacles_manager.configuration as configuration
import octobot_tentacles_manager.constants as constants
import octobot_tentacles_manager.util as util
import octobot_tentacles_manager.managers as managers


async def ensure_setup_configuration(tentacle_path=constants.TENTACLES_PATH, bot_path=constants.DEFAULT_BOT_PATH,
                                     bot_install_dir=constants.DEFAULT_BOT_INSTALL_DIR) -> None:
    if not path.exists(path.join(bot_path, constants.USER_REFERENCE_TENTACLE_CONFIG_FILE_PATH)):
        await api.repair_installation(tentacle_path, bot_path, bot_install_dir, verbose=False)


def refresh_profile_tentacles_setup_config(
        profile_folder,
        tentacle_path=constants.TENTACLES_PATH,
        bot_installation_path=constants.DEFAULT_BOT_PATH,
        bot_install_dir=constants.DEFAULT_BOT_INSTALL_DIR
):
    tentacles_setup_manager = managers.TentaclesSetupManager(path.join(bot_installation_path, tentacle_path),
                                                             bot_installation_path,
                                                             path.join(bot_install_dir,
                                                                       constants.DEFAULT_TENTACLE_CONFIG))
    tentacles_setup_manager.refresh_profile_tentacles_config(profile_folder)


def get_tentacles_setup_config(
    config_path=constants.USER_REFERENCE_TENTACLE_CONFIG_FILE_PATH
) -> configuration.TentaclesSetupConfiguration:
    setup_config = configuration.TentaclesSetupConfiguration(config_path=config_path)
    setup_config.read_config()
    return setup_config


def create_tentacles_setup_config_with_tentacles(
    *tentacles_classes, config_path=constants.USER_REFERENCE_TENTACLE_CONFIG_FILE_PATH
) -> configuration.TentaclesSetupConfiguration:
    setup_config = configuration.TentaclesSetupConfiguration(config_path=config_path)
    setup_config.from_activated_tentacles_classes(*tentacles_classes)
    return setup_config


def fill_with_installed_tentacles(
    tentacles_setup_config: configuration.TentaclesSetupConfiguration,
    tentacles_folder: str = constants.TENTACLES_PATH
):
    available_tentacle = util.load_tentacle_with_metadata(tentacles_folder)
    # fill overall tentacles setup config data
    tentacles_setup_config.fill_tentacle_config(
        available_tentacle,
    )


def is_tentacle_activated_in_tentacles_setup_config(tentacles_setup_config: configuration.TentaclesSetupConfiguration,
                                                    klass_name: str, default_value=False, raise_errors=False) -> bool:
    try:
        return tentacles_setup_config.is_tentacle_activated(klass_name)
    except KeyError as e:
        if raise_errors:
            raise e
        return default_value


def get_class_from_name_with_activated_required_tentacles(name: str,
                                                          parent_class,
                                                          tentacles_setup_config: configuration.TentaclesSetupConfiguration,
                                                          with_class_method=None):
    for subclass in tentacles_management.get_all_classes_from_parent(parent_class):
        # Filter sub classes to only use the one that is appropriate to the given name and that has its
        # tentacles requirements activated (identified by REQUIRED_ACTIVATED_TENTACLES iterable class attribute).
        if subclass.get_name() == name and \
                tentacles_setup_config is not None and \
                all(is_tentacle_activated_in_tentacles_setup_config(tentacles_setup_config, required_tentacle.__name__)
                    for required_tentacle in subclass.REQUIRED_ACTIVATED_TENTACLES):
            if with_class_method is None or getattr(subclass, with_class_method)():
                return subclass
    return None


def get_tentacles_activation(tentacles_setup_config: configuration.TentaclesSetupConfiguration) -> dict:
    return tentacles_setup_config.tentacles_activation


def get_tentacles_config_folder(tentacles_setup_config: configuration.TentaclesSetupConfiguration) -> str:
    return tentacles_setup_config.get_config_folder()


def get_registered_tentacle_packages(tentacles_setup_config: configuration.TentaclesSetupConfiguration) -> dict:
    return tentacles_setup_config.registered_tentacles


def unregister_tentacle_packages(tentacles_setup_config: configuration.TentaclesSetupConfiguration,
                                 package_name: str) -> dict:
    return tentacles_setup_config.unregister_tentacles_package(package_name)


def update_activation_configuration(tentacles_setup_config: configuration.TentaclesSetupConfiguration,
                                    new_config: dict, deactivate_others: bool, add_missing_elements=False) \
        -> bool:
    return tentacles_setup_config.update_activation_configuration(new_config, deactivate_others, add_missing_elements)


def deactivate_all_tentacles(tentacles_setup_config: configuration.TentaclesSetupConfiguration,
                             trading_modes: bool, strategies: bool, evaluators: bool):
    tentacles_setup_config.deactivate_all(trading_modes, strategies, evaluators)


def save_tentacles_setup_configuration(tentacles_setup_config: configuration.TentaclesSetupConfiguration) -> None:
    tentacles_setup_config.save_config()


def has_profile_local_configuration(tentacles_setup_config: configuration.TentaclesSetupConfiguration, tentacle_class):
    return path.exists(configuration.get_profile_config_specific_file_path(tentacles_setup_config, tentacle_class))


def get_activated_tentacles(tentacles_setup_config: configuration.TentaclesSetupConfiguration) -> list:
    return [tentacle_class
            for tentacle_classes in get_tentacles_activation(tentacles_setup_config).values()
            for tentacle_class, activated in tentacle_classes.items()
            if activated]


def update_tentacle_config(tentacles_setup_config: configuration.TentaclesSetupConfiguration,
                           tentacle_class: object, config_data: dict, keep_existing: bool=True) -> None:
    configuration.update_config(tentacles_setup_config, tentacle_class, config_data, keep_existing)


def import_user_tentacles_config_folder(tentacles_setup_config: configuration.TentaclesSetupConfiguration):
    # WARNING: this code should only be called on profiles that are safe to import (might be a security break otherwise)
    # => careful with downloaded profiles
    user_tentacles_config_folder = configuration.get_user_tentacles_config_folder(tentacles_setup_config)
    if path.isdir(user_tentacles_config_folder) and user_tentacles_config_folder not in sys.path:
        sys.path.append(user_tentacles_config_folder)


def get_tentacle_config(tentacles_setup_config: configuration.TentaclesSetupConfiguration, klass) -> dict:
    return configuration.get_config(tentacles_setup_config, klass)


def set_tentacle_config_proxy(new_proxy) -> dict:
    return configuration.set_get_config_proxy(new_proxy)


def local_tentacle_config_proxy(new_proxy):
    return configuration.local_get_config_proxy(new_proxy)


def factory_tentacle_reset_config(tentacles_setup_config: configuration.TentaclesSetupConfiguration, klass) -> None:
    return configuration.factory_reset_config(tentacles_setup_config, klass)


def get_tentacle_config_schema_path(klass) -> str:
    return configuration.get_config_schema_path(klass)


def get_compiled_tentacles_url(base_url, version) -> str:
    return f"{base_url}{util.get_local_arch_download_path()}/{version}.zip"


def set_tentacles_setup_configuration_path(tentacles_setup_config: configuration.TentaclesSetupConfiguration, new_path):
    tentacles_setup_config.config_path = new_path


def get_code_hash(tentacles: list) -> str:
    return util.get_tentacles_code_hash(tentacles)


def get_config_hash(tentacles: list,
                    tentacles_setup_config: configuration.TentaclesSetupConfiguration) -> str:
    return util.get_tentacles_config_hash(tentacles, tentacles_setup_config)



#  Drakkar-Software OctoBot-Tentacles-Manager
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import octobot_tentacles_manager.enums as enums
import octobot_tentacles_manager.constants as constants
import octobot_tentacles_manager.models.tentacle_package as tentacle_package_model
import octobot_tentacles_manager.models.metadata.artifact_metadata as artifact_metadata


class TentaclePackageMetadata(artifact_metadata.ArtifactMetadata):
    def __init__(self, artifact: tentacle_package_model.TentaclePackage):
        super().__init__(artifact)
        self.artifact_type: enums.ArtifactTypes = enums.ArtifactTypes.TENTACLE_PACKAGE
        self.original_metadata_dict = None

    def load_from_dict(self) -> None:
        if self.original_metadata_dict:
            if self.original_metadata_dict.get(constants.ARTIFACT_METADATA_VERSION):
                self.artifact.version = self.original_metadata_dict.get(constants.ARTIFACT_METADATA_VERSION)

    def to_dict(self) -> dict:
        origin_dict = super().to_dict()
        origin_dict[constants.ARTIFACT_METADATA_TENTACLES] = [
            artifact.get_name_with_version() for artifact in self.artifact.artifacts
        ]
        if self.original_metadata_dict:
            origin_dict.update(self.original_metadata_dict)
        return origin_dict



#  Drakkar-Software OctoBot-Tentacles-Manager
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.

from octobot_tentacles_manager.api import configurator
from octobot_tentacles_manager.api import updater
from octobot_tentacles_manager.api import loader
from octobot_tentacles_manager.api import inspector
from octobot_tentacles_manager.api import uninstaller
from octobot_tentacles_manager.api import creator
from octobot_tentacles_manager.api import installer
from octobot_tentacles_manager.api import util
from octobot_tentacles_manager.api import uploader

from octobot_tentacles_manager.api.util import (
    manage_tentacles,
)

from octobot_tentacles_manager.api.uploader import (
    upload_file_or_folder,
    upload_file_to_nexus,
    upload_folder_to_nexus,
    upload_file_or_folder_to_nexus,
    upload_file_to_s3,
    upload_folder_to_s3,
    upload_file_or_folder_to_s3,
)

from octobot_tentacles_manager.api.configurator import (
    get_tentacles_setup_config,
    create_tentacles_setup_config_with_tentacles,
    fill_with_installed_tentacles,
    is_tentacle_activated_in_tentacles_setup_config,
    get_class_from_name_with_activated_required_tentacles,
    get_tentacles_activation,
    get_registered_tentacle_packages,
    unregister_tentacle_packages,
    update_activation_configuration,
    deactivate_all_tentacles,
    save_tentacles_setup_configuration,
    has_profile_local_configuration,
    get_activated_tentacles,
    update_tentacle_config,
    import_user_tentacles_config_folder,
    get_tentacle_config,
    set_tentacle_config_proxy,
    local_tentacle_config_proxy,
    factory_tentacle_reset_config,
    get_tentacle_config_schema_path,
    get_compiled_tentacles_url,
    set_tentacles_setup_configuration_path,
    ensure_setup_configuration,
    refresh_profile_tentacles_setup_config,
    get_code_hash,
    get_config_hash,
)
from octobot_tentacles_manager.api.updater import (
    update_all_tentacles,
    update_tentacles,
)
from octobot_tentacles_manager.api.loader import (
    load_tentacles,
    is_tentacles_architecture_valid,
    reload_tentacle_info,
    ensure_tentacle_info,
    register_extra_tentacle_data,
    are_tentacles_up_to_date,
    is_tentacles_setup_config_successfully_loaded,
    get_tentacles_installation_version,
)
from octobot_tentacles_manager.api.inspector import (
    get_installed_tentacles_modules,
    get_tentacle_group,
    get_tentacle_version,
    get_tentacle_origin_package,
    get_tentacle_module_name,
    get_tentacle_classes_requirements,
    get_tentacle_resources_path,
    get_tentacle_documentation_path,
    get_tentacle_documentation,
    check_tentacle_version,
    get_tentacle_class_from_string,
    get_tentacles_classes_names_for_type,
    get_installed_packages_from_url,
    get_tentacles_from_package_name,
    get_all_installed_package_urls,
)
from octobot_tentacles_manager.api.uninstaller import (
    uninstall_all_tentacles,
    uninstall_tentacles,
)
from octobot_tentacles_manager.api.creator import (
    start_tentacle_creator,
    create_tentacles_package,
    create_all_tentacles_bundle,
)
from octobot_tentacles_manager.api.installer import (
    install_all_tentacles,
    install_tentacles,
    install_single_tentacle,
    repair_installation,
)

__all__ = [
    "get_tentacles_setup_config",
    "create_tentacles_setup_config_with_tentacles",
    "fill_with_installed_tentacles",
    "is_tentacle_activated_in_tentacles_setup_config",
    "get_class_from_name_with_activated_required_tentacles",
    "get_tentacles_activation",
    "get_registered_tentacle_packages",
    "unregister_tentacle_packages",
    "update_activation_configuration",
    "deactivate_all_tentacles",
    "save_tentacles_setup_configuration",
    "has_profile_local_configuration",
    "get_activated_tentacles",
    "update_tentacle_config",
    "get_tentacle_config",
    "set_tentacle_config_proxy",
    "local_tentacle_config_proxy",
    "import_user_tentacles_config_folder",
    "factory_tentacle_reset_config",
    "get_tentacle_config_schema_path",
    "get_compiled_tentacles_url",
    "set_tentacles_setup_configuration_path",
    "get_code_hash",
    "get_config_hash",
    "ensure_setup_configuration",
    "refresh_profile_tentacles_setup_config",
    "update_all_tentacles",
    "update_tentacles",
    "load_tentacles",
    "is_tentacles_architecture_valid",
    "reload_tentacle_info",
    "ensure_tentacle_info",
    "register_extra_tentacle_data",
    "are_tentacles_up_to_date",
    "is_tentacles_setup_config_successfully_loaded",
    "get_tentacles_installation_version",
    "get_installed_tentacles_modules",
    "get_tentacle_group",
    "get_tentacle_version",
    "get_tentacle_origin_package",
    "get_tentacle_module_name",
    "get_tentacle_classes_requirements",
    "get_tentacle_resources_path",
    "get_tentacle_documentation_path",
    "get_tentacle_documentation",
    "check_tentacle_version",
    "get_tentacle_class_from_string",
    "get_tentacles_classes_names_for_type",
    "get_installed_packages_from_url",
    "get_tentacles_from_package_name",
    "get_all_installed_package_urls",
    "uninstall_all_tentacles",
    "uninstall_tentacles",
    "start_tentacle_creator",
    "create_tentacles_package",
    "install_all_tentacles",
    "install_tentacles",
    "install_single_tentacle",
    "repair_installation",
    "manage_tentacles",
    "create_all_tentacles_bundle",
    "upload_file_or_folder",
    "upload_file_to_nexus",
    "upload_folder_to_nexus",
    "upload_file_or_folder_to_nexus",
    "upload_file_to_s3",
    "upload_folder_to_s3",
    "upload_file_or_folder_to_s3",
]



#  Drakkar-Software OctoBot-Commons
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import json
import inspect
import hashlib

import octobot_tentacles_manager.configuration.tentacle_configuration as tentacle_configuration


def get_tentacles_code_hash(tentacles: list) -> str:
    full_code = ""
    for linked_tentacle in tentacles:
        code_location = (
            linked_tentacle.get_script()
            if (
                hasattr(linked_tentacle, "get_script")
                and linked_tentacle.TRADING_SCRIPT_MODULE
            )
            else linked_tentacle.__class__
        )
        full_code = f"{full_code}{inspect.getsource(code_location)}"
    return hashlib.sha256(full_code.encode()).hexdigest()


def get_tentacles_config_hash(identifying_tentacles: list, tentacles_setup_config) -> str:
    full_config = ""
    for linked_tentacle in identifying_tentacles:
        config = linked_tentacle.specific_config if hasattr(linked_tentacle, "specific_config") \
            and linked_tentacle.specific_config else \
            tentacle_configuration.get_config(tentacles_setup_config, linked_tentacle.__class__)
        full_config = f"{full_config}{json.dumps(config)}"
    return hashlib.sha256(full_config.encode()).hexdigest()
